#!/usr/bin/python

import sys, os, re
from distutils.core import setup, Command
from distutils.extension import Extension
from distutils.errors import *
from os.path import exists

MAJOR = 2
MINOR = 0

def main():

    version = None
    revision = latest_revision('.')
    if revision:
        version = '%s.%s.%s' % (MAJOR, MINOR, revision)
    else:
        print 'WARNING: Building from a non-Subversion directory or cannot run svnversion.  Using version 0.0.0'

    files     = [ 'pyodbcmodule.cpp', 'cursor.cpp', 'row.cpp', 'connection.cpp', 'buffer.cpp', 'params.cpp', 'errors.cpp',
                  'getdata.cpp' ]
    libraries = []

    extra_compile_args = None
    extra_link_args    = None

    if os.name == 'nt':
        # Windows native
        files.append('pyodbc.rc')
        libraries.append('odbc32')
        extra_compile_args = [ '/W4' ]

        # Add debugging symbols
        extra_compile_args = [ '/W4', '/Zi', '/Od' ]
        extra_link_args    = [ '/DEBUG' ]

    elif os.environ.get("OS", '').lower().startswith('windows'):
        # Windows Cygwin (posix on windows)
        # OS name not windows, but still on Windows
        libraries.append('odbc32')

    elif sys.platform == 'darwin':
        # OS/X now ships with iODBC.
        libraries.append('iodbc')

    else:
        # Other posix-like: Linux, Solaris, etc.
        # What is the proper way to detect iODBC, MyODBC, unixODBC, etc.?
        libraries.append('odbc')

    if exists('MANIFEST'):
        os.remove('MANIFEST')

    setup (name = "pyodbc",
           version = version,
           description = "DB API %s.%s Module for ODBC" % (MAJOR, MINOR),

           long_description = ('A Python DB API 2 module for ODBC. This project provides an up-to-date, '
                               'convenient interface to ODBC using native data types like datetime and decimal.'),

           maintainer       = "Michael Kleehammer",
           maintainer_email = "mkleehammer@users.sourceforge.net",

           ext_modules = [ Extension('pyodbc', files,
                                     libraries=libraries,
                                     define_macros=[('PYODBC_REVISION', str(revision))],
                                     extra_compile_args=extra_compile_args,
                                     extra_link_args=extra_link_args
                                     ) ],

           classifiers = [ 'Development Status :: 5 - Production/Stable',
                           'Intended Audience :: Developers',
                           'Intended Audience :: System Administrators',
                           'License :: OSI Approved :: MIT License',
                           'Operating System :: Microsoft :: Windows',
                           'Operating System :: POSIX',
                           'Programming Language :: Python',
                           'Topic :: Database',
                           ],

           url = 'http://pyodbc.sourceforge.net',
           download_url = 'https://sourceforge.net/project/showfiles.php?group_id=162557',
           cmdclass = dict(tags=TagsCommand, apitest=APITestCommand))


class TagsCommand(Command):
    description = 'Create a TAGS file for emacs and vim using etags.'

    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if not self.dry_run:
            os.system('etags *.cpp *.h')


    
class APITestCommand(Command):

    description = 'Run the dbapi20 unittests.'

    user_options = [ ('connection-string=', 'c', 'ODBC connection string'),
                     ('verbose=', 'v', 'Run verbose tests'), ]

    def initialize_options(self):
        self.connection_string = None

    def finalize_options(self):
        if not self.connection_string:
            # Find right exception
            raise DistutilsOptionError('Missing requird connection-string option.')

    def run(self):
        if not self.dry_run:
            add_to_path()
            import pyodbc
            print 'pyodbc version:', pyodbc.version

            import dbapi20, unittest, pyodbc

            cs = self.connection_string

            class test_pyodbc(dbapi20.DatabaseAPI20Test):
                driver = pyodbc
                connect_args = [ cs ]
                connect_kw_args = {}

                def test_nextset(self): pass
                def test_setoutputsize(self): pass
                def test_ExceptionsAsConnectionAttributes(self): pass

            suite = unittest.makeSuite(test_pyodbc, 'test')
            testRunner = unittest.TextTestRunner(verbosity=(self.verbose > 1) and 9 or 0)
            result = testRunner.run(suite)


def getoutput(cmd):
    import os
    pipe = os.popen(cmd, 'r')
    text = pipe.read()
    sts = pipe.close()
    if sts is None: sts = 0
    if text[-1:] == '\n': text = text[:-1]
    return sts, text

def latest_revision(path):

    status, output = getoutput('svnversion -cn %s' % path)

    # If this directory is not a Subversion directory, the English svn will print 'exported'.  Other languages print
    # other things.  To simplify, we assume it is not if the value doesn't start with a number e.g. 4123:4168
    output = output.strip()

    if not output or not re.match('\d.*', output):
        # We are not in a Subversion directory.  If this was built from a source distribution, try to get the version
        # from the PKG-INFO file that 'setup sdist' adds.

        from os.path import exists
        if exists('PKG-INFO'):
            contents = file('PKG-INFO').read()
            m = re.search(r'Version: \s+ \d+ \. \d+ \. (\d+)', contents, re.VERBOSE)
            if m:
                output = m.group(1)

    if not output:
        raise SystemExit('Unable to determine the revision number.')

    if ':' in output:
        output = output.split(':', 1)[1]

    if output[-1] == 'M':
        output = output[:-1]

    return int(output)


def add_to_path():
    """
    Prepends the build directory to the path so that newly built pyodbc libraries are used, allowing it to be tested
    without installing it.
    """
    # Put the build directory into the Python path so we pick up the version we just built.
    #
    # To make this cross platform, we'll search the directories until we find the .pyd file.

    import imp

    library_exts  = [ t[0] for t in imp.get_suffixes() if t[-1] == imp.C_EXTENSION ]
    library_names = [ 'pyodbc%s' % ext for ext in library_exts ]

    # Only go into directories that match our version number. 

    dir_suffix = '-%s.%s' % (sys.version_info[0], sys.version_info[1])

    for root, dirs, files in os.walk('build'):
        for d in dirs[:]:
            if not d.endswith(dir_suffix):
                dirs.remove(d)

        for name in library_names:
            if name in files:
                sys.path.insert(0, root)
                return
                
    raise SystemExit('Unable to find the pyodbc library in the build directory.  Have you built it?')


if __name__ == '__main__':
    main()
